@@ -1,5 +1,5 @@
 /*  This file is part of "choosewm"
- *  Copyright (C) 2005 Bernhard R. Link
+ *  Copyright (C) 2005,2009 Bernhard R. Link
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
  *  published by the Free Software Foundation.
@@ -17,6 +17,8 @@
 
 #include <stdlib.h>
 #include <malloc.h>
+#include <string.h>
+#include <unistd.h>
 #include <getopt.h>
 #define GTK_DISABLE_DEPRECATED 1
 #include <gtk/gtk.h>
@@ -27,14 +29,14 @@
 /*#include <locale.h>
 #include <libintl.h>*/
 
-GtkWidget *window;
-GtkWidget *checker;
-GtkTreeSelection *wmselection;
-int returncode = EXIT_FAILURE;
-gboolean configmode = false;
-gboolean printmode = false;
-gboolean askagain = true;
-gint wmnumber = -1;
+static GtkWidget *window;
+static GtkWidget *checker;
+static GtkTreeSelection *wmselection;
+static gboolean textmode = false;
+static gboolean configmode = false;
+static gboolean printmode = false;
+static gboolean askagain = true;
+static gint wmnumber = -1;
 
 #define STRINGS S(CONFIGWELCOME), S(WELCOME), S(CONFIGHEADER), S(HEADER), S(CONFIGASK), S(ASK), S(CONFIGSELECT), S(SELECT), S(CONFIGCANCEL), S(CANCEL)
 const char *stringkeys[] = {
@@ -99,22 +101,21 @@ GtkTreeIter defaultiter;
 
 static void makeWMItems(GtkListStore *store) {
 	struct windowmanager *wm;
-	gint wmnumber = 0;
+	gint number = 0;
 
-	for( wm = windowmanagers; wm != NULL ; wm = wm->next,wmnumber++ ) {
+	for( wm = windowmanagers; wm != NULL ; wm = wm->next,number++ ) {
 		GtkTreeIter iter;
 
 		gtk_list_store_append(store,&iter);
 		gtk_list_store_set(store,&iter,
 			col_name,wm->name,
-			col_number,wmnumber,
+			col_number,number,
 			-1);
 		if( wm->isdefault ) {
 			defaultiter = iter;
 			defaultavailable = TRUE;
 		}
 	}
-
 }
 
 static gint wmsortfunc(GtkTreeModel *model, GtkTreeIter  *item1, GtkTreeIter  *item2, gpointer      user_data) {
@@ -240,10 +241,12 @@ static void makeWindowContent(GtkContainer *container) {
 
 int main(int argc, char *argv[]){
 	const struct option longopts[] = {
+		{"text",no_argument,NULL,'t'},
 		{"print",no_argument,NULL,'p'},
 		{"debug",no_argument,NULL,'d'},
 		{"force",no_argument,NULL,'f'},
 		{"config",no_argument,NULL,'c'},
+		{"help",no_argument,NULL,'h'},
 		{NULL,no_argument,NULL,0}
 	};
 	int opt;
@@ -256,8 +259,20 @@ int main(int argc, char *argv[]){
 		return EXIT_FAILURE;
 	}*/
 	inittrings();
-	while( (opt = getopt_long(argc,argv,"dcfp",longopts,NULL)) != -1 ) {
+	while( (opt = getopt_long(argc,argv,"dcfph",longopts,NULL)) != -1 ) {
 		switch( opt ) {
+			case 'h':
+				puts(
+"choosewm - select a window manager to be used\n"
+"Syntax: choosewm [--force] [--print|--config]\n"
+" --text:	Ask in terminal\n"
+" --config:	Do not start but only change setting\n"
+" --print:	Do not start but print\n"
+" --force:	Always ask\n");
+				return EXIT_SUCCESS;
+			case 't':
+				textmode = TRUE;
+				break;
 			case 'd':
 				break;
 			case 'c':
@@ -276,6 +291,16 @@ int main(int argc, char *argv[]){
 				exit(EXIT_FAILURE);
 		}
 	}
+	if( textmode && !isatty(0) ) {
+		fprintf(stderr, "%s: ERROR: No terminal for --textmode!\n",
+				argv[0]);
+		return EXIT_FAILURE;
+	}
+	if( configmode && getenv("DISPLAY") == NULL && !isatty(0) ) {
+		fprintf(stderr, "%s: ERROR: No terminal and no DISPLAY set!\n",
+				argv[0]);
+		return EXIT_FAILURE;
+	}
 	readconfigs(stringkeys,stringvalues);
 	askagain = !dontask;
 	if( haderrors )
@@ -294,36 +319,79 @@ int main(int argc, char *argv[]){
 		}
 	}
 
-	gtk_init(&argc, &argv);
+	if( textmode || (getenv("DISPLAY") == NULL && configmode) ) {
+		struct windowmanager *wm;
+		char buffer[100], *s, *e;
+		int i = 0, j;
 
-	window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
-	g_signal_connect(G_OBJECT(window),
-			"delete_event",
-			G_CALLBACK(delete_event), NULL);
-	gtk_container_set_border_width(GTK_CONTAINER(window), 10);
-	makeWindowContent(GTK_CONTAINER(window));
-
-	GTK_WIDGET_SET_FLAGS(startbutton, GTK_CAN_DEFAULT);
-	GTK_WIDGET_SET_FLAGS(startbutton, GTK_RECEIVES_DEFAULT);
-	gtk_widget_grab_default(startbutton);
-
-	screen = gtk_window_get_screen(GTK_WINDOW(window));
-
-	if( configmode ) {
-		gtk_window_maximize(GTK_WINDOW(window));
+		puts("Please select your default WindowManager:");
+		for( wm = windowmanagers; wm != NULL ; wm = wm->next ) {
+			printf("%c%2d: %s\n", wm->isdefault?'*':' ',
+					i++, wm->name);
+		}
+		fputs("Your choice? ", stdout);
+		if( fgets(buffer, 99, stdin) == NULL )
+			return EXIT_FAILURE;
+		s = buffer + strlen(buffer);
+		while( s > buffer && (*(s-1) == '\n' || *(s-1) == '\r'
+					|| *(s-1) == ' ' || *(s-1) == '\t') )
+			*(--s) = '\0';
+		s = buffer;
+		while( *s == ' ' )
+			s++;
+		j = strtol(s, &e, 10);
+		if( *e != '\0' && *e != '*' ) {
+			fprintf(stderr, "%s: Error parsing '%s'\n", argv[0], s);
+			return EXIT_FAILURE;
+		}
+		if( s == e || *e == '*' ) {
+			wm = windowmanagers;
+			wmnumber = 0;
+		        while( wm != NULL && !wm->isdefault ) {
+				wm = wm->next;
+				wmnumber++;
+			}
+			if( wm == NULL ) {
+				fputs("ERROR: no default window manager\n", stderr);
+				return EXIT_FAILURE;
+			}
+		} else if( j < 0 || j >= i ) {
+			fprintf(stderr, "ERROR: Not a valid choice: %d\n", j);
+			return EXIT_FAILURE;
+		} else
+			wmnumber = j;
 	} else {
-		/* gtk_window_fullscreen does not work when there
-		 * is not yet a window manager, so make the window
-		 * as large as possible manually. (This both makes
-		 * sure the list is properly navigateable and the
-		 * keys work without any mouse-moving or key-grabbing */
-		gtk_window_set_gravity(GTK_WINDOW(window),GDK_GRAVITY_STATIC);
-		gtk_window_move(GTK_WINDOW(window),0,0);
-		gtk_window_resize(GTK_WINDOW(window),gdk_screen_get_width(screen),gdk_screen_get_height(screen));
-	}
-	gtk_widget_show(window);
+		gtk_init(&argc, &argv);
 
-	gtk_main();
+		window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
+		g_signal_connect(G_OBJECT(window),
+				"delete_event",
+				G_CALLBACK(delete_event), NULL);
+		gtk_container_set_border_width(GTK_CONTAINER(window), 10);
+		makeWindowContent(GTK_CONTAINER(window));
+
+		GTK_WIDGET_SET_FLAGS(startbutton, GTK_CAN_DEFAULT);
+		GTK_WIDGET_SET_FLAGS(startbutton, GTK_RECEIVES_DEFAULT);
+		gtk_widget_grab_default(startbutton);
+
+		screen = gtk_window_get_screen(GTK_WINDOW(window));
+
+		if( configmode ) {
+			gtk_window_maximize(GTK_WINDOW(window));
+		} else {
+			/* gtk_window_fullscreen does not work when there
+			 * is not yet a window manager, so make the window
+			 * as large as possible manually. (This both makes
+			 * sure the list is properly navigateable and the
+			 * keys work without any mouse-moving or key-grabbing */
+			gtk_window_set_gravity(GTK_WINDOW(window),GDK_GRAVITY_STATIC);
+			gtk_window_move(GTK_WINDOW(window),0,0);
+			gtk_window_resize(GTK_WINDOW(window),gdk_screen_get_width(screen),gdk_screen_get_height(screen));
+		}
+		gtk_widget_show(window);
+
+		gtk_main();
+	}
 	if( wmnumber >= 0) {
 		struct windowmanager *wm = windowmanagers;
 		while( wmnumber > 0 ) {
